(* file: gccontent.ml
 * author: Bob Muller
 * material from CS1101, Boston College
 *
 * Compute the GC content of a string of nucleotides.
 *
 * See: http://rosalind.info/problems/gc/
 * 
 * Compile: > ocamlc -o gccontent gccontent.ml
 * Run:     > ./gccontent a.fas
 *)
let explode s =
  let rec repeat i acc =
    match i < 0 with
    | true  -> acc
    | false -> repeat (i - 1) (s.[i]::acc)
  in
  repeat (String.length s - 1) []

let rec findMax pairs =
  match pairs with
  | [] -> failwith "findMax: error, empty list"
  | [pair] -> pair
  | (id1, p1)::(id2, p2)::rest ->
     match p1 > p2 with
     | true  -> findMax ((id1, p1)::rest)
     | false -> findMax ((id2, p2)::rest)

type fasta = {id:string; nucs:string}
	       
let countGCs strand =
  let nucs = explode (String.uppercase strand) in
  let rec repeat nucs count =
    match nucs with
    | [] -> count   
    | 'G'::nucs | 'C'::nucs -> repeat nucs (count + 1)
    | _::nucs -> repeat nucs count
  in
  repeat nucs 0
	
let gcPercentage nucs =
  let counts = float (countGCs nucs) in
  let n = float (String.length nucs)
  in
  counts /. n

let readFASTA file =
  let inch = open_in file in
  let rec repeat fasta =
    try
      let id = input_line inch in
      let nucs = input_line inch
      in
      repeat ({id=id; nucs=nucs}::fasta)
    with
      End_of_file -> close_in inch;
		     fasta
  in
  repeat []
      
let () =
  let file   = Sys.argv.(1) in
  let fasta  = readFASTA file in
  let pairs  = List.map (fun {id; nucs} -> (id, gcPercentage nucs)) fasta in
  let (id, percentage) = findMax pairs
  in
  print_endline id;
  print_float percentage;
  print_endline ""
