(* Now for input values a and b, what is the area of under the the
   curve in the interval f(a) to f(b).  We can approximate this
   using the Riemann method: by inscribing N rectangles and summing 
   their areas.
 *)
let closeEnough a b = abs_float(a -. b) < 1e-8

(* val leftRiemann : (float -> float) -> float -> float -> int -> float

   The call (leftRieman f a b n) divides the area between a and b
   into n equal intervals. It returns the sum of the areas of those
   rectangles thereby approximating the area under the curve.

   NB: As n grows larger the rectangles become narrower and the
       approximation is closer to the definite integral of f between
       a and b.
 *)
let leftRiemann f a b n =
  let dx = (b -. a) /. (float n) in
  let rec repeat c acc =
    match (closeEnough c b) with
    | true  -> acc
    | false -> let h = f c
	       in
	       repeat (c +. dx) (acc +. h)
  in
  dx *. (repeat a 0.0)
